/* ddutil.c : Fonctions DirectDraw trs pratiques */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <ddraw.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdarg.h>
#include "debug.h"
#include "dd.h"
#include "jpeg/jpeg.h"
#include "main.h"


// Retourne une chaine de caractres correspondant  une erreur
static char *DDErrorString(HRESULT hr)
{
	switch (hr)
	{
	case DDERR_ALREADYINITIALIZED:           return "DDERR_ALREADYINITIALIZED";
	case DDERR_CANNOTATTACHSURFACE:          return "DDERR_CANNOTATTACHSURFACE";
	case DDERR_CANNOTDETACHSURFACE:          return "DDERR_CANNOTDETACHSURFACE";
	case DDERR_CURRENTLYNOTAVAIL:            return "DDERR_CURRENTLYNOTAVAIL";
	case DDERR_EXCEPTION:                    return "DDERR_EXCEPTION";
	case DDERR_GENERIC:                      return "DDERR_GENERIC";
	case DDERR_HEIGHTALIGN:                  return "DDERR_HEIGHTALIGN";
	case DDERR_INCOMPATIBLEPRIMARY:          return "DDERR_INCOMPATIBLEPRIMARY";
	case DDERR_INVALIDCAPS:                  return "DDERR_INVALIDCAPS";
	case DDERR_INVALIDCLIPLIST:              return "DDERR_INVALIDCLIPLIST";
	case DDERR_INVALIDMODE:                  return "DDERR_INVALIDMODE";
	case DDERR_INVALIDOBJECT:                return "DDERR_INVALIDOBJECT";
	case DDERR_INVALIDPARAMS:                return "DDERR_INVALIDPARAMS";
	case DDERR_INVALIDPIXELFORMAT:           return "DDERR_INVALIDPIXELFORMAT";
	case DDERR_INVALIDRECT:                  return "DDERR_INVALIDRECT";
	case DDERR_LOCKEDSURFACES:               return "DDERR_LOCKEDSURFACES";
	case DDERR_NO3D:                         return "DDERR_NO3D";
	case DDERR_NOALPHAHW:                    return "DDERR_NOALPHAHW";
	case DDERR_NOCLIPLIST:                   return "DDERR_NOCLIPLIST";
	case DDERR_NOCOLORCONVHW:                return "DDERR_NOCOLORCONVHW";
	case DDERR_NOCOOPERATIVELEVELSET:        return "DDERR_NOCOOPERATIVELEVELSET";
	case DDERR_NOCOLORKEY:                   return "DDERR_NOCOLORKEY";
	case DDERR_NOCOLORKEYHW:                 return "DDERR_NOCOLORKEYHW";
	case DDERR_NODIRECTDRAWSUPPORT:          return "DDERR_NODIRECTDRAWSUPPORT";
	case DDERR_NOEXCLUSIVEMODE:              return "DDERR_NOEXCLUSIVEMODE";
	case DDERR_NOFLIPHW:                     return "DDERR_NOFLIPHW";
	case DDERR_NOGDI:                        return "DDERR_NOGDI";
	case DDERR_NOMIRRORHW:                   return "DDERR_NOMIRRORHW";
	case DDERR_NOTFOUND:                     return "DDERR_NOTFOUND";
	case DDERR_NOOVERLAYHW:                  return "DDERR_NOOVERLAYHW";
	case DDERR_NORASTEROPHW:                 return "DDERR_NORASTEROPHW";
	case DDERR_NOROTATIONHW:                 return "DDERR_NOROTATIONHW";
	case DDERR_NOSTRETCHHW:                  return "DDERR_NOSTRETCHHW";
	case DDERR_NOT4BITCOLOR:                 return "DDERR_NOT4BITCOLOR";
	case DDERR_NOT4BITCOLORINDEX:            return "DDERR_NOT4BITCOLORINDEX";
	case DDERR_NOT8BITCOLOR:                 return "DDERR_NOT8BITCOLOR";
	case DDERR_NOTEXTUREHW:                  return "DDERR_NOTEXTUREHW";
	case DDERR_NOVSYNCHW:                    return "DDERR_NOVSYNCHW";
	case DDERR_NOZBUFFERHW:                  return "DDERR_NOZBUFFERHW";
	case DDERR_NOZOVERLAYHW:                 return "DDERR_NOZOVERLAYHW";
	case DDERR_OUTOFCAPS:                    return "DDERR_OUTOFCAPS";
	case DDERR_OUTOFMEMORY:                  return "DDERR_OUTOFMEMORY";
	case DDERR_OUTOFVIDEOMEMORY:             return "DDERR_OUTOFVIDEOMEMORY";
	case DDERR_OVERLAYCANTCLIP:              return "DDERR_OVERLAYCANTCLIP";
	case DDERR_OVERLAYCOLORKEYONLYONEACTIVE: return "DDERR_OVERLAYCOLORKEYONLYONEACTIVE";
	case DDERR_PALETTEBUSY:                  return "DDERR_PALETTEBUSY";
	case DDERR_COLORKEYNOTSET:               return "DDERR_COLORKEYNOTSET";
	case DDERR_SURFACEALREADYATTACHED:       return "DDERR_SURFACEALREADYATTACHED";
	case DDERR_SURFACEALREADYDEPENDENT:      return "DDERR_SURFACEALREADYDEPENDENT";
	case DDERR_SURFACEBUSY:                  return "DDERR_SURFACEBUSY";
	case DDERR_CANTLOCKSURFACE:              return "DDERR_CANTLOCKSURFACE";
	case DDERR_SURFACEISOBSCURED:            return "DDERR_SURFACEISOBSCURED";
	case DDERR_SURFACELOST:                  return "DDERR_SURFACELOST";
	case DDERR_SURFACENOTATTACHED:           return "DDERR_SURFACENOTATTACHED";
	case DDERR_TOOBIGHEIGHT:                 return "DDERR_TOOBIGHEIGHT";
	case DDERR_TOOBIGSIZE:                   return "DDERR_TOOBIGSIZE";
	case DDERR_TOOBIGWIDTH:                  return "DDERR_TOOBIGWIDTH";
	case DDERR_UNSUPPORTED:                  return "DDERR_UNSUPPORTED";
	case DDERR_UNSUPPORTEDFORMAT:            return "DDERR_UNSUPPORTEDFORMAT";
	case DDERR_UNSUPPORTEDMASK:              return "DDERR_UNSUPPORTEDMASK";
	case DDERR_VERTICALBLANKINPROGRESS:      return "DDERR_VERTICALBLANKINPROGRESS";
	case DDERR_WASSTILLDRAWING:              return "DDERR_WASSTILLDRAWING";
	case DDERR_XALIGN:                       return "DDERR_XALIGN";
	case DDERR_INVALIDDIRECTDRAWGUID:        return "DDERR_INVALIDDIRECTDRAWGUID";
	case DDERR_DIRECTDRAWALREADYCREATED:     return "DDERR_DIRECTDRAWALREADYCREATED";
	case DDERR_NODIRECTDRAWHW:               return "DDERR_NODIRECTDRAWHW";
	case DDERR_PRIMARYSURFACEALREADYEXISTS:  return "DDERR_PRIMARYSURFACEALREADYEXISTS";
	case DDERR_NOEMULATION:                  return "DDERR_NOEMULATION";
	case DDERR_REGIONTOOSMALL:               return "DDERR_REGIONTOOSMALL";
	case DDERR_CLIPPERISUSINGHWND:           return "DDERR_CLIPPERISUSINGHWND";
	case DDERR_NOCLIPPERATTACHED:            return "DDERR_NOCLIPPERATTACHED";
	case DDERR_NOHWND:                       return "DDERR_NOHWND";
	case DDERR_HWNDSUBCLASSED:               return "DDERR_HWNDSUBCLASSED";
	case DDERR_HWNDALREADYSET:               return "DDERR_HWNDALREADYSET";
	case DDERR_NOPALETTEATTACHED:            return "DDERR_NOPALETTEATTACHED";
	case DDERR_NOPALETTEHW:                  return "DDERR_NOPALETTEHW";
	case DDERR_BLTFASTCANTCLIP:              return "DDERR_BLTFASTCANTCLIP";
	case DDERR_NOBLTHW:                      return "DDERR_NOBLTHW";
	case DDERR_NODDROPSHW:                   return "DDERR_NODDROPSHW";
	case DDERR_OVERLAYNOTVISIBLE:            return "DDERR_OVERLAYNOTVISIBLE";
	case DDERR_NOOVERLAYDEST:                return "DDERR_NOOVERLAYDEST";
	case DDERR_INVALIDPOSITION:              return "DDERR_INVALIDPOSITION";
	case DDERR_NOTAOVERLAYSURFACE:           return "DDERR_NOTAOVERLAYSURFACE";
	case DDERR_EXCLUSIVEMODEALREADYSET:      return "DDERR_EXCLUSIVEMODEALREADYSET";
	case DDERR_NOTFLIPPABLE:                 return "DDERR_NOTFLIPPABLE";
	case DDERR_CANTDUPLICATE:                return "DDERR_CANTDUPLICATE";
	case DDERR_NOTLOCKED:                    return "DDERR_NOTLOCKED";
	case DDERR_CANTCREATEDC:                 return "DDERR_CANTCREATEDC";
	case DDERR_NODC:                         return "DDERR_NODC";
	case DDERR_WRONGMODE:                    return "DDERR_WRONGMODE";
	case DDERR_IMPLICITLYCREATED:            return "DDERR_IMPLICITLYCREATED";
	case DDERR_NOTPALETTIZED:                return "DDERR_NOTPALETTIZED";
	case DDERR_UNSUPPORTEDMODE:              return "DDERR_UNSUPPORTEDMODE";
	case DDERR_NOMIPMAPHW:                   return "DDERR_NOMIPMAPHW";
	case DDERR_INVALIDSURFACETYPE:           return "DDERR_INVALIDSURFACETYPE";
	case DDERR_DCALREADYCREATED:             return "DDERR_DCALREADYCREATED";
	case DDERR_CANTPAGELOCK:                 return "DDERR_CANTPAGELOCK";
	case DDERR_CANTPAGEUNLOCK:               return "DDERR_CANTPAGEUNLOCK";
	case DDERR_NOTPAGELOCKED:                return "DDERR_NOTPAGELOCKED";
	case DDERR_NOTINITIALIZED:               return "DDERR_NOTINITIALIZED";
	}
	return "Unknown Error";
}


// Vrifie un rsultat DirectDraw
BOOL DDCheck(HRESULT hr, const char *format, ... )
{
	if( FAILED(hr) )
	{
		char buf[1024];
		va_list args;

		va_start(args, format);
		_vsnprintf(buf, sizeof(buf), format, args );
		va_end(args);
		Error("DirectDraw : %s (%s)", buf, DDErrorString(hr) );
		
		return FALSE;
	}
	return TRUE;
}


// Fonction DDColorMatch(): converti les couleurs RGB
static DWORD DDColorMatch( LPDIRECTDRAWSURFACE lpDDS, COLORREF rgb )
{
	COLORREF				rgbT;
	HDC						hdc;
	DWORD					dw = CLR_INVALID;
	DDSURFACEDESC			ddsd;
    HRESULT					hr;

	if (rgb != CLR_INVALID && IDirectDrawSurface_GetDC( lpDDS, &hdc) == DD_OK)
	{
		rgbT = GetPixel(hdc,0,0); // Sauvegarde de la valeur du Pixel actuel
		SetPixel(hdc,0,0,rgb);    // Met notre valeur
		IDirectDrawSurface_ReleaseDC(lpDDS, hdc);
	}
	
	// On lock la surfrace pour lire la couleur convertie

	ddsd.dwSize = sizeof(ddsd);
	while ((hr = IDirectDrawSurface_Lock(lpDDS, NULL, &ddsd,0,NULL)) == DDERR_WASSTILLDRAWING);
	if (hr == DD_OK)
	{
		dw=*(DWORD *) ddsd.lpSurface; //Obtiens le DWORD
		if (ddsd.ddpfPixelFormat.dwRGBBitCount <32)
			dw &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount) -1; //Le cacher en bbp
		IDirectDrawSurface_Unlock(lpDDS,NULL);
	}
	//On remet la couleur qui y tait

	if (rgb!= CLR_INVALID && IDirectDrawSurface_GetDC(lpDDS, &hdc)==DD_OK)
	{
		SetPixel(hdc,0,0,rgbT);
		IDirectDrawSurface_ReleaseDC(lpDDS, hdc);
	}
	return dw;
}


// Fonction DDSetColorKey(): Met une color key  une surface, RGB.
//Prends en compte le pixel du haut  gauche.
static HRESULT DDSetColorKey( LPDIRECTDRAWSURFACE lpDDS, COLORREF rgb )
{
	DDCOLORKEY	ddck;

	ddck.dwColorSpaceLowValue = DDColorMatch(lpDDS, rgb);
	ddck.dwColorSpaceHighValue = ddck.dwColorSpaceLowValue;
	return IDirectDrawSurface_SetColorKey( lpDDS, DDCKEY_SRCBLT, &ddck );
}


//Dessine un bitmap sur une surface DD
static BOOL PaintBitmap(LPDIRECTDRAWSURFACE lpDDS, HBITMAP hbm, int iWidth, int iHeight)
{
	HDC	hdc;
	HDC hdcComp = CreateCompatibleDC(hdc);

	if((!hbm)||(!lpDDS))
		return FALSE;
	
	// Vrouillage du back-buffer, hdc=hdc du back buffer
	if(!DDCheck(IDirectDrawSurface_GetDC(lpDDS, &hdc),"DDLoadJpeg : PaintBitmap : GetDC"))
		return FALSE;
	
	// Affichage de l'image
	SelectBitmap(hdcComp,hbm);
	BitBlt(hdc,0,0,iWidth,iHeight,hdcComp,0,0,SRCCOPY);
	DeleteDC(hdcComp);
	
	// Dverrouilage du back buffer
	if(!DDCheck(IDirectDrawSurface_ReleaseDC(lpDDS,hdc),"DDLoadJpeg : PaintBitmap : ReleaseDC"))
		return FALSE;
	
	return TRUE;
}

//Cre une surface DD
static LPDIRECTDRAWSURFACE CreateSurface(int iWidth, int iHeight)
{
	HRESULT hr;
	DDSURFACEDESC ddsd;
	LPDIRECTDRAWSURFACE lpDDS;

    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_CKSRCBLT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth = iWidth;
    ddsd.dwHeight = iHeight;
	
    hr = IDirectDraw_CreateSurface( g_lpDD, &ddsd, &lpDDS, NULL);
	if(!DDCheck(hr, "DDLoadJpeg : CreateSurface : Can't create surface"))
	{
        return NULL;
	}
	
	return lpDDS;
}

/* LoadJPGImage : Charge une image JPEG depuis un fichier ou une resource
 *
 *	Args:
 *    -szName (IN) : resource or file name
 *    -piWidth (OUT) : image width
 *    -piHeight (OUT) : image height
 *
 *  Return:
 *	  handle to new bitmap object (must be freed by DeleteBitmap)
 */
static HBITMAP LoadJPGImage( LPSTR szName, int *piWidth, int *piHeight )
{
	int					iWidth,
						iHeight,
						iBpp;
	void				*pBuf;
	HDC					hdc;
	BITMAPINFOHEADER	bmih;
	BITMAPINFO          *pBmi;
	DWORD				dwSize;
	HGLOBAL				hmemBuf;
	HBITMAP				hbm;
	HRSRC				res;
	HGLOBAL				hResData;
	char				*buf;
	
	//Essaye de trouver la resource
	res = FindResource( g_hInstance, szName, "JPEG" );
	if(res == NULL) //Resource non trouve: szName doit tre un nom de fichier
	{
		//Dcode le fichier Jpeg (JPG->BMP)
		if(!JpegLoadFromFile(szName,&hmemBuf,&iWidth,&iHeight,&iBpp,&dwSize))
		{
			return NULL;
		}
	}
	else			//szName est un nom de resource
	{
		//Charge la ressource
		hResData = LoadResource(g_hInstance,res);
		if(hResData == NULL)
			return NULL;
		buf = (char*)LockResource(hResData);
		//Dcode la resource Jpeg (JPG->BMP)
		if(!JpegLoadFromBuffer(buf,SizeofResource(g_hInstance,res),&hmemBuf,&iWidth,&iHeight,
			&iBpp,&dwSize))
		{
			return NULL;
		}
	}
	
	//Vrifie qu'il ne s'agit pas d'une image en niveaux de gris
	if(iBpp == 8 )
	{
		Error("DDLoadJpeg : LoadJPGImage : 8 bits image (grayscale), cannot be loaded (%s)",szName);
		GlobalFree(hmemBuf);
		return NULL;
	}
	
	//Permet de rcuprer la taille aprs l'appel  la fonction
	*piWidth = iWidth;
	*piHeight = iHeight;
	
	//Obtient le HDC de la surface back buffer (pour la convertion de la palette)
	if(!DDCheck(IDirectDrawSurface_GetDC(g_lpDDSBack,&hdc),"DDLoadJpeg : LoadJPGImage : GetDC"))
	{
		GlobalFree(hmemBuf);
		return NULL;
	}
	
	//Obtient un pointeur sur les donnes de l'image
	pBuf=GlobalLock(hmemBuf);
	
	// "Conversion" de l'image en DIB : cre un bitmap  partir des donnes de l'image
	ZeroMemory(&bmih,sizeof(bmih));
	bmih.biSize=sizeof(bmih);
	bmih.biWidth=iWidth;
	bmih.biHeight=-(LONG)iHeight;	// <0=image "normale"
	bmih.biPlanes=1;
	bmih.biBitCount=(WORD)iBpp;
	bmih.biCompression=BI_RGB;
	bmih.biClrUsed=(iBpp==8?256:0);
	
	pBmi=(BITMAPINFO*)malloc(sizeof(BITMAPINFOHEADER));
	if(!pBmi)
	{
		Error("DDLoadJpeg : LoadJPGImage : malloc : out of memory");
		GlobalUnlock(hmemBuf);
		GlobalFree(hmemBuf);
		return NULL;
	}
	CopyMemory(pBmi,&bmih,sizeof(bmih));
	
	hbm = CreateDIBitmap(hdc,&bmih,CBM_INIT,pBuf,pBmi,iBpp==8?DIB_RGB_COLORS:0);
	
	free(pBmi);
	
	//Relache le device context
	DDCheck(IDirectDrawSurface_ReleaseDC(g_lpDDSBack,hdc),"DDLoadJpeg : LoadJPGImage : ReleaseDC");
	
	//Dcharge la memoire des donnes de l'image qui sont dsormais inutiles
	GlobalUnlock(hmemBuf);
	GlobalFree(hmemBuf);
	
	return hbm;
}

//Charge une resource ou un fichier JPEG et cre une surface  partir de celui-ci
LPDIRECTDRAWSURFACE DDLoadJpeg( LPCSTR szName )
{
	HBITMAP hbm;
	int iWidth, iHeight;
	LPDIRECTDRAWSURFACE lpDDS;
	HRESULT hr;
	
	//Charge l'image
	hbm = LoadJPGImage( (char*)szName, &iWidth, &iHeight );
	if(!hbm)
		return NULL;
	
	//Cre la surface
	lpDDS = CreateSurface(iWidth,iHeight);
	if(!lpDDS)
	{
		DeleteBitmap(hbm);
		return NULL;
	}
	
	//Copie l'image sur la surface
	if(!PaintBitmap(lpDDS,hbm,iWidth,iHeight))
	{
		DeleteBitmap(hbm);
		IDirectDrawSurface_Release( lpDDS );
		return NULL;
	}
	
	//Supprime le bitmap
	DeleteBitmap(hbm);
	
	//Configure la color key
	hr = DDSetColorKey(lpDDS, CLR_INVALID);
	if(!DDCheck(hr, "DDLoadJpeg : Can't set color key"))
	{
		IDirectDrawSurface_Release( lpDDS );
		return NULL;
	}
	
	return lpDDS;
}


/*
 *  DDLoadBitmap
 *
 *  create a DirectDrawSurface from a bitmap file or resource.
 *
 */
LPDIRECTDRAWSURFACE DDLoadBitmap( LPCSTR szBitmap, int width, int height)
{
	HBITMAP             hbm;
	BITMAP              bm;
	DDSURFACEDESC       ddsd;
	LPDIRECTDRAWSURFACE lpDDS = NULL;
	HDC					hdcImage = NULL;
	HDC					hdcSurf = NULL;
	HRESULT				hr;
	
    //
    //  try to load the bitmap as a resource, if that fails, try it as a file
    //
    hbm = (HBITMAP) LoadImage(GetModuleHandle(NULL), 
		szBitmap, 
		IMAGE_BITMAP, 
		width, 
		height, 
		LR_CREATEDIBSECTION);
	
    if (hbm == NULL)
        hbm = (HBITMAP)LoadImage(NULL, szBitmap, IMAGE_BITMAP, width, height, LR_LOADFROMFILE|LR_CREATEDIBSECTION);

    if (hbm == NULL)
	{
		Error("DDLoadBitmap : LoadImage : Can't load image(%s)",szBitmap);
        return NULL;
	}
	
    //
    // get size of the bitmap
    //
    GetObject(hbm, sizeof(bm), &bm);      // get size of bitmap
	
    //
    // create a DirectDrawSurface for this bitmap
    //
    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_CKSRCBLT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth = bm.bmWidth;
    ddsd.dwHeight = bm.bmHeight;
	
    hr = IDirectDraw_CreateSurface(g_lpDD, &ddsd, &lpDDS, NULL);
	if(!DDCheck(hr, "DDLoadBitmap : Can't create surface"))
	{
		DeleteObject(hbm);
        return NULL;
	}
	
	// Cre un DC et y slectionne l'image
	hdcImage = CreateCompatibleDC(NULL);
	SelectObject(hdcImage, hbm);
	
	//Obtient un DC pour la surface
	hr = IDirectDrawSurface_GetDC( lpDDS, &hdcSurf );
	if(!DDCheck(hr, "DDLoadBitmap : Can't get DC"))
	{
		DeleteDC(hdcImage);
		DeleteObject(hbm);
		IDirectDrawSurface_Release( lpDDS );
		return NULL;
	}
	
	//BitBlt convertira le format si ncessaire.
	if (BitBlt (hdcSurf, 0,0,ddsd.dwWidth, ddsd.dwHeight, hdcImage, 0, 0, SRCCOPY) == FALSE)
	{
		Error("DDLoadBitmap : BitBlt");
		DeleteDC(hdcImage);
		DeleteObject(hbm);
		IDirectDrawSurface_ReleaseDC( lpDDS, hdcSurf );
		IDirectDrawSurface_Release( lpDDS );
		return NULL;
	}
	
	DeleteDC(hdcImage);
	DeleteObject(hbm);
	IDirectDrawSurface_ReleaseDC( lpDDS, hdcSurf );
	
	hr = DDSetColorKey(lpDDS, CLR_INVALID);
	if(!DDCheck(hr, "DDLoadBitmap : Can't set color key"))
	{
		IDirectDrawSurface_Release( lpDDS );
		return NULL;
	}

	
    return lpDDS;
}
