/* di.c : fonctions DirectInput non spcifiques au jeu */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <dinput.h>
#include "diutil.h"
#include "debug.h"


//Objets direct input
LPDIRECTINPUT			g_lpDI = NULL;
LPDIRECTINPUTDEVICE		g_lpDIDev = NULL;


//Acquire
void DIAcquire(void)
{
	if( g_lpDIDev )
	{
		DICheck( IDirectInputDevice_Acquire( g_lpDIDev ),"DIAcquire : Acquire");
	}
}


//Unacquire
void DIUnacquire(void)
{
	if( g_lpDIDev )
	{
		DICheck( IDirectInputDevice_Unacquire( g_lpDIDev ),"DIUnacquire : Unacquire");
	}
}


//Initialise le clavier
static BOOL InitKeyboard( HWND hWnd )
{
	//Create the DirectInput Keyboard Device
	
/*	if(!DICheck(
		IDirectInput7_CreateDeviceEx
								(
								g_lpDI,
								&GUID_SysKeyboard,
								&IID_IDirectInputDevice7,
								(void**)&g_lpDIDev,
								NULL
								),
		"Create DirectInput Keyboard Device"
		))*/
	if(!DICheck(
			IDirectInput_CreateDevice
									(
									g_lpDI,
									&GUID_SysKeyboard,
									&g_lpDIDev,
									NULL
									),
			"DIInit : InitKeyboard : CreateDevice : Create DirectInput Keyboard Device"
		))
	{
		IDirectInput_Release( g_lpDI );
		g_lpDI = NULL;
		return FALSE;
	}
	
	//Set the Keyboard Data Format
	
	if(!DICheck(
		IDirectInputDevice_SetDataFormat( g_lpDIDev, &c_dfDIKeyboard ),
		"DIInit : InitKeyboard : SetDataFormat : Set the Keyboard Data Format"
		))
	{
		IDirectInputDevice_Release( g_lpDIDev );
		g_lpDIDev = NULL;
		IDirectInput_Release( g_lpDI );
		g_lpDI = NULL;
		return FALSE;
	}
	
	// Set the cooperative level
	
	if(!DICheck(
		IDirectInputDevice_SetCooperativeLevel( g_lpDIDev, hWnd,
										DISCL_FOREGROUND | DISCL_NONEXCLUSIVE),
		"DIInit : InitKeyboard : SetCooperativeLevel : Set the cooperative level"
		))
	{
		IDirectInputDevice_Release( g_lpDIDev );
		g_lpDIDev = NULL;
		IDirectInput_Release( g_lpDI );
		g_lpDI = NULL;
		return FALSE;
	}
	
	//Gain Access to the Keyboard
	
	if(!DICheck(
		IDirectInputDevice_Acquire( g_lpDIDev ),
		"DIInit : InitKeyboard : Acquire : Gain Access to the Keyboard"
		))
	{
		IDirectInputDevice_Release( g_lpDIDev );
		g_lpDIDev = NULL;
		IDirectInput_Release( g_lpDI );
		g_lpDI = NULL;
		return FALSE;
	}
	
	return TRUE;
}


//Initialise DirectInput
BOOL DIInit( HINSTANCE h, HWND hWnd )
{
	//Create the DirectInputObject
	HRESULT hr;
	
/*	if(!DICheck(
		DirectInputCreateEx(h,
							DIRECTINPUT_VERSION,
							&IID_IDirectInput7,
							(void**)&g_lpDI,
							NULL),
		"Create DirectInputObject"
		))
		return FALSE;*/
	hr = DirectInputCreate( h, DIRECTINPUT_VERSION, &g_lpDI, NULL );
	if(hr == DIERR_OLDDIRECTINPUTVERSION)
	{
		DWORD dwVersion;
		dwVersion = DIRECTINPUT_VERSION - 0x0100;
		Debug("DIInit : DirectInputCreate : Version %x no found, Testing version (%x)",DIRECTINPUT_VERSION,dwVersion);
		while((hr = DirectInputCreate( h, dwVersion, &g_lpDI, NULL )) == DIERR_OLDDIRECTINPUTVERSION)
		{
			dwVersion -= 0x0100;
			Debug("DIInit : DirectInputCreate : Testing version (%x)",dwVersion);
		}
	}
	if(!DICheck( hr,
				"DIInit : DirectInputCreate : Create DirectInputObject"
		))
		return FALSE;
	
	return InitKeyboard( hWnd );
}


//Ferme le device (keyboard)
static void CloseDevice(void)
{
	if (g_lpDIDev)
	{
		/* Always unacquire device before calling Release(). */
		IDirectInputDevice_Unacquire( g_lpDIDev );
		IDirectInputDevice_Release( g_lpDIDev );
		g_lpDIDev = NULL;
	}
}


//Ferme DirectInput
void DIClose(void) 
{
	if (g_lpDI)
	{
		CloseDevice();
		IDirectInput_Release( g_lpDI );
		g_lpDI = NULL;
	}
}
