/* drawscreen.c : Dessine l'cran */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <stdio.h>
#include <ddraw.h>
#include "sprites.h"
#include "ddutil.h"
#include "dd.h"
#include "game.h"
#include "debug.h"
#include "metrics.h"


//Les blocks de tetris
static void DrawBlocks(void)
{
	HRESULT hr;
	RECT rcDest;
	int i,j,b;

	for( i = 0 ; i < 10 ; i++ )
	{
		for( j = 0 ; j < 18 ; j++ )
		{
			b = g_blocks2draw[i][j];
			if(b)
			{
#ifdef _DEBUG
				if((b<1)||(b>7))
					Error("DrawBlocks : invalid block number (%i)", b);
#endif //_DEBUG
				SetRect( &rcDest,
					i * g_metricsBlockW + g_metricsBlockX, j * g_metricsBlockH + g_metricsBlockY,
					(i+1) * g_metricsBlockW + g_metricsBlockX, (j+1) * g_metricsBlockH + g_metricsBlockY );
				hr = IDirectDrawSurface_Blt(
									g_lpDDSBack,
									&rcDest,
									g_lpDDSBlocks[b-1],
									NULL,
									DDBLT_WAIT,
									NULL );	
				DDCheck(hr, "DrawBlocks : Blt g_lpDDSBlocks[%i]", b);
			}
		}
	}
}


//La prochaine pice  venir
static void DrawNext(void)
{
	HRESULT hr;
	RECT rcDest;
	int i,j,b;

	for( i = 0 ; i < 4 ; i++ )
	{
		for( j = 0 ; j < 4 ; j++ )
		{
			b = g_next[i][j];
			if(b)
			{
#ifdef _DEBUG
				if((b<1)||(b>7))
					Error("DrawNext : invalid block number (%i)", b);
#endif //_DEBUG
				SetRect( &rcDest,
					i * g_metricsBlockW + g_metricsNextX, j * g_metricsBlockH + g_metricsNextY,
					(i+1) * g_metricsBlockW + g_metricsNextX, (j+1) * g_metricsBlockH + g_metricsNextY );
/*				SetRect( &rcDest, i * BLOCK_W + NEXT_X, j * BLOCK_H + NEXT_Y,
						(i+1) * BLOCK_W + NEXT_X, (j+1) * BLOCK_H + NEXT_Y );*/
				hr = IDirectDrawSurface_Blt(
									g_lpDDSBack,
									&rcDest,
									g_lpDDSBlocks[b-1],
									NULL,
									DDBLT_WAIT,
									NULL );	
				DDCheck(hr, "DrawNext : Blt g_lpDDSBlocks[%i]", b);
			}
		}
	}
}


//Le dessin de fond
static void DrawBackground(void)
{
	HRESULT hr;
	
	hr = IDirectDrawSurface_Blt(
						g_lpDDSBack,
						NULL,
						g_lpDDSBackground,
						NULL,
						DDBLT_WAIT,
						NULL );
	
	DDCheck(hr, "DrawScreen : Blt g_lpDDSBackground");
}


//Dessine le nombre 'number' aux coordonnes 'x' et 'y' spcifies
static void DrawNumber(int x, int y, int number)
{
	HRESULT hr;
	RECT rcDest;
	char n;
	char buf[6];
	int i, t = 0;
	
	_snprintf( buf, sizeof(buf), "%.5d", number );
	for( i = 0 ; i < 5 ; i++ )
	{
		n = buf[i] - 48;
		if((n==0)&&(!t)&&(i<4))
		{
			continue;
		}
		else
		{
			t = 1;
		}
		SetRect( &rcDest, x + i * g_metricsNumberW, y, x + (i+1) * g_metricsNumberW, y + g_metricsNumberH );
		hr = IDirectDrawSurface_Blt(
				g_lpDDSBack,
				&rcDest,
				g_lpDDSNumber[n],
				NULL,
				DDBLT_KEYSRC|DDBLT_WAIT,
				NULL );
		DDCheck(hr, "DrawNumber : Blt g_lpDDSNumber[%i]", n);
	}
}


//Dessine le score, le nombre de lignes, le meilleur score, le niveau et la vitesse
static void DrawScore(void)
{
	DrawNumber(g_metricsScoreX,g_metricsScoreY,g_score);
	DrawNumber(g_metricsLinesX,g_metricsLinesY,g_lines);
	DrawNumber(g_metricsBestX,g_metricsBestY,g_best);
	DrawNumber(g_metricsLevelX,g_metricsLevelY,g_level);
	DrawNumber(g_metricsSpeedX,g_metricsSpeedY,g_speed);
}


//Ecrit 'Pause'
static void DrawPause(void)
{
	HRESULT hr;
	RECT rcDest;
	
	SetRect(&rcDest, g_metricsPauseX, g_metricsPauseY, g_metricsPauseX + g_metricsPauseW,
		g_metricsPauseY + g_metricsPauseH );
	hr = IDirectDrawSurface_Blt(
						g_lpDDSBack,
						&rcDest,
						g_lpDDSPause,
						NULL,
						DDBLT_KEYSRC|DDBLT_WAIT,
						NULL );
	
	DDCheck(hr, "DrawPause : Blt g_lpDDSPause");
}


// dessine tout l'cran
void DrawScreen(void)
{
	DrawBackground();
	DrawBlocks();
	DrawNext();
	DrawScore();
	if(g_bPause)
		DrawPause();
}
