/* dsutil.c : Fonctions DirectSound trs pratiques */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <dsound.h>
#include <stdarg.h>
#include <stdio.h>
#include "ds.h"
#include "wave.h"
#include "mpeg/mpeg.h"
#include "debug.h"


#define PCM_BUFFER_SIZE	100000


// Vrifie un rsultat retourn par une fonction DirectSound
BOOL DSCheck(HRESULT hr, const char *format, ... )
{
	if( hr == DS_OK ) return TRUE;
	else
	{
		char buf[1024];
		va_list args;
		char *er;
		va_start(args, format);
		_vsnprintf(buf, sizeof(buf), format, args);
		va_end(args);
		switch(hr)
		{
		case DSERR_ACCESSDENIED:		er = ("DSERR_ACCESSDENIED");		break;
		case DSERR_ALLOCATED:			er = ("DSERR_ALLOCATED");			break;
		case DSERR_ALREADYINITIALIZED:	er = ("DSERR_ALREADYINITIALIZED");	break;
		case DSERR_BADFORMAT:			er = ("DSERR_BADFORMAT");			break;
		case DSERR_BUFFERLOST:			er = ("DSERR_BUFFERLOST");			break;
		case DSERR_CONTROLUNAVAIL:		er = ("DSERR_CONTROLUNAVAIL");		break;
		case DSERR_GENERIC:				er = ("DSERR_GENERIC");				break;
		case DSERR_INVALIDCALL:			er = ("DSERR_INVALIDCALL");			break;
		case DSERR_INVALIDPARAM:		er = ("DSERR_INVALIDPARAM");		break;
		case DSERR_NOAGGREGATION:		er = ("DSERR_NOAGGREGATION");		break;
		case DSERR_NODRIVER:			er = ("DSERR_NODRIVER");			break;
		case DSERR_NOINTERFACE:			er = ("DSERR_NOINTERFACE");			break;
		case DSERR_OTHERAPPHASPRIO:		er = ("DSERR_OTHERAPPHASPRIO");		break;
		case DSERR_OUTOFMEMORY:			er = ("DSERR_OUTOFMEMORY");			break;
		case DSERR_PRIOLEVELNEEDED:		er = ("DSERR_PRIOLEVELNEEDED");		break;
		case DSERR_UNINITIALIZED:		er = ("DSERR_UNINITIALIZED");		break;
		case DSERR_UNSUPPORTED:			er = ("DSERR_UNSUPPORTED");			break;
		default:
			er = ("Unknown error");
		}
		Error("DirectSound : %s (%s)",buf,er);
	}
	return FALSE;
}


// Joue le son contenu dans un buffer DSound
BOOL DSPlay(LPDIRECTSOUNDBUFFER lpDSB )
{
	HRESULT	hr;

	if( lpDSB == NULL )
	{
		return TRUE;
	}

	hr = IDirectSoundBuffer_SetCurrentPosition( lpDSB, 0 );
	if(!DSCheck(hr, "DSPlay : SetCurrentPosition"))
		return FALSE;

	hr = IDirectSoundBuffer_Play( lpDSB, 0, 0, 0 );
	if(!DSCheck(hr, "DSPlay : Play"))
		return FALSE;

	return TRUE;
}


// Charge un fichier .WAV  partir d'un fichier ou d'une ressource qui se trouve dans 
//l'excutable. Retourne un buffer DSound
LPDIRECTSOUNDBUFFER DSLoadWave( LPSTR lpName )
{
	HRESULT				hr;
	DSBUFFERDESC		dsbdesc;
	LPDIRECTSOUNDBUFFER lpDSB = NULL;
	WAVEFORMATEX		*pwfx;				// Wave format info.
	HMMIO				hmmio;				// File handle.
	MMCKINFO			mmckinfo;			// Chunk info.
	MMCKINFO			mmckinfoParent;		// Parent chunk info.
	LPVOID				lpvAudio1;
	DWORD				dwBytes1;
	UINT				cbBytesRead;		//Number of bytes read
	
	//Open the file
	if ( WaveOpenFile( lpName, &hmmio, &pwfx, &mmckinfoParent ) != 0 )
	{
		Error("DSLoadWave : WaveOpenFile");
		return NULL;
	}
	
	//Read the file
	if ( WaveStartDataRead( &hmmio, &mmckinfo, &mmckinfoParent ) != 0 )
	{
		Error("DSLoadWave : WaveStartDataRead");
		GlobalFree(pwfx);
		return NULL;
	}
	
	//Create the buffer
	ZeroMemory( &dsbdesc, sizeof(DSBUFFERDESC) );
	dsbdesc.dwSize = sizeof(DSBUFFERDESC);
	dsbdesc.dwFlags = DSBCAPS_STATIC;
	dsbdesc.dwBufferBytes = mmckinfo.cksize;
	dsbdesc.dwReserved = 0;
	dsbdesc.lpwfxFormat = pwfx; 
	
	hr = IDirectSound_CreateSoundBuffer( g_lpDS, &dsbdesc, &lpDSB, NULL );
	if(!DSCheck(hr, "DSLoadWave : CreateBuffer"))
	{
		WaveCloseReadFile( &hmmio, &pwfx );
		return NULL;
	}
	
	//Lock the buffer
	hr = IDirectSoundBuffer_Lock(
		lpDSB,
		0,						// Offset of lock start.
		0,						// Size of lock; ignored in this case.
		&lpvAudio1,				// Address of lock start.
		&dwBytes1,				// Number of bytes locked.
		NULL,					// Address of wraparound start.
		NULL,					// Number of wraparound bytes.
		DSBLOCK_ENTIREBUFFER	// Flag.
		);
	if(!DSCheck(hr, "LoadSound : Lock"))
	{
		// Error handling.
		WaveCloseReadFile( &hmmio, &pwfx );
		return NULL;
	}
	
	
	if ( WaveReadFile( hmmio,	// File handle.
		dwBytes1,				// Number of bytes to read.
		( BYTE * )lpvAudio1,	// Destination.
		&mmckinfo,				// File chunk info.
		&cbBytesRead ) )		// Actual number of bytes read.
	{
		// Handle failure on nonzero return.
		WaveCloseReadFile( &hmmio, &pwfx );
		Error("LoadSound : WaveReadFile");
		return NULL;
    }
	
	//unlock the buffer
    IDirectSoundBuffer_Unlock( lpDSB, lpvAudio1, dwBytes1, NULL, 0 );
	
	//close the file
	WaveCloseReadFile( &hmmio, &pwfx );
	
	return lpDSB;
}


//Cre un buffer DirectSound  partir d'un fichier MPEG (.mp3)
LPDIRECTSOUNDBUFFER DSLoadMpeg( LPSTR szName )
{
	LPDIRECTSOUNDBUFFER lpDSB;
	DSBUFFERDESC dsbdesc;
	HRESULT hr;
	DWORD dwBufferLen;
	WAVEFORMATEX wfx;
	LPVOID lpvAudio1;
	DWORD dwBytes1;
	DEC_INFO decinfo;
	HANDLE hFile;
	DWORD dwFileSize, dwNbRead;
	BYTE *pBuf;
	MPEG_HEAD head;
	int framebytes;
	IN_OUT x;
	char *pcm_buffer;
	int bs_bytes,pcm_bytes;
	
	// Read whole file
	hFile = CreateFile(szName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(hFile==INVALID_HANDLE_VALUE)
	{
		return NULL;
	}

	dwFileSize = GetFileSize(hFile,NULL);
	pBuf = (BYTE*)malloc(dwFileSize);
	if(!pBuf)
	{
		Error("DSLoadMpeg : malloc");
		CloseHandle(hFile);
		return NULL;
	}

	if(!ReadFile(hFile,pBuf,dwFileSize,&dwNbRead,NULL))
	{
		Error("DSLoadMpeg : ReadFile");
		CloseHandle(hFile);
		free(pBuf);
		return NULL;
	}

	if(dwNbRead!=dwFileSize)
	{
		Error("DSLoadMpeg : ReadFile : dwNbRead != dwFileSize");
		CloseHandle(hFile);
		free(pBuf);
		return NULL;
	}
	
	//Get header
	framebytes = head_info(pBuf,dwFileSize,&head);
	if(!framebytes)
	{
		Error("DSLoadMpeg : head_info : BAD OR UNSUPPORTED MPEG FILE");
		CloseHandle(hFile);
		free(pBuf);
		return NULL;
	}
	
	//Init decoder
	if (!audio_decode_init(&head, framebytes, 0, 0, 0, 24000))
	{
		Error("DSLoadMpeg : audio.decode_init");
		CloseHandle(hFile);
		free(pBuf);
		return NULL;
	}
	
	//Get format
	audio_decode_info(&decinfo);
	
	wfx.wFormatTag = WAVE_FORMAT_PCM;
	wfx.nChannels = decinfo.channels;
	wfx.nSamplesPerSec = decinfo.samprate;
	wfx.wBitsPerSample = decinfo.bits;
	wfx.nBlockAlign = wfx.nChannels * (wfx.wBitsPerSample / 8);
	wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
	wfx.cbSize = 0;
	
	//Decode
	pcm_buffer = (char*)malloc(PCM_BUFFER_SIZE);
	if(!pcm_buffer)
	{
		Error("DSLoadMpeg : pcm_buffer : malloc(%i)",PCM_BUFFER_SIZE);
		CloseHandle(hFile);
		free(pBuf);
		return NULL;
	}
	
	bs_bytes = 0;
	pcm_bytes = 0;
	
	do
	{
		x = audio_decode((pBuf+bs_bytes), (short*)(pcm_buffer+pcm_bytes));
		bs_bytes += x.in_bytes;
		pcm_bytes += x.out_bytes;
		if(pcm_bytes>PCM_BUFFER_SIZE-10000)
		{
			Error("DSLoadMpeg : decode : pcm buffer too small (%i)",PCM_BUFFER_SIZE);
			CloseHandle(hFile);
			free(pBuf);
			free(pcm_buffer);
			return NULL;
		}
	}while(x.in_bytes>0);
	
	dwBufferLen = pcm_bytes;
	
	CloseHandle(hFile);
	free(pBuf);
	
	//Create the buffer
	ZeroMemory( &dsbdesc, sizeof(DSBUFFERDESC) );
	dsbdesc.dwSize = sizeof(DSBUFFERDESC);
	dsbdesc.dwFlags = DSBCAPS_STATIC;
	dsbdesc.dwBufferBytes = dwBufferLen;
	dsbdesc.lpwfxFormat = &wfx; 
	
	hr = IDirectSound_CreateSoundBuffer( g_lpDS, &dsbdesc, &lpDSB, NULL );
	if(!DSCheck(hr, "DSLoadMpeg : CreateBuffer"))
	{
		free(pcm_buffer);
		return NULL;
	}
	
	//Lock the buffer
	hr = IDirectSoundBuffer_Lock(
		lpDSB,
		0,						// Offset of lock start.
		0,						// Size of lock; ignored in this case.
		&lpvAudio1,				// Address of lock start.
		&dwBytes1,				// Number of bytes locked.
		NULL,					// Address of wraparound start.
		NULL,					// Number of wraparound bytes.
		DSBLOCK_ENTIREBUFFER	// Flag.
		);
	if(!DSCheck(hr, "DSLoadMpeg : Lock"))
	{
		IDirectSoundBuffer_Release(lpDSB);
		free(pcm_buffer);
		return NULL;
	}
	
	if(dwBufferLen!=dwBytes1)
	{
		Error("dwBufferLen(%i)!=dwBytes1(%i)",dwBufferLen,dwBytes1);
	}
	memcpy( lpvAudio1, pcm_buffer, dwBufferLen );
	
	free(pcm_buffer);
	
	//unlock the buffer
    if(!DSCheck(IDirectSoundBuffer_Unlock( lpDSB, lpvAudio1, dwBytes1, NULL, 0 ),
		"DSLoadMpeg : Unlock"))
	{
		IDirectSoundBuffer_Release(lpDSB);
		return NULL;
	}
	
	return lpDSB;
}
