/* main.c : Fonctions WinMain, WindowProc et SwitchDisplayMode */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <windows.h>

#ifndef SetWindowStyle	//Fonction non prsente dans windowsx.h
#define SetWindowStyle(hwnd,dw) SetWindowLong(hwnd,GWL_STYLE,dw)
#endif //SetWindowStyle

#include "dd.h"
#include "ds.h"
#include "di.h"
#include "debug.h"
#include "winutil.h"
#include "loadingwindow.h"
#include "config.h"
#include "game.h"
#include "sprites.h"
#include "sounds.h"
#include "key.h"
#include "skins.h"


#define NAME	"UltimateTetrisWndClass"	// Le nom de la classe
#define TITLE	"Ultimate Tetris"			// Le titre de la fenetre


HMODULE g_hInstance = NULL;		// handle du programme
HWND g_hWnd = NULL;				// handle de la fenetre
RECT g_rcWindow;				// rectangle de la fenetre
BOOL g_bActive = FALSE;			// est-ce que le jeu est actif?


//Change Full Screen/Windowed mode
BOOL SwitchDisplayMode(void)
{
	UnloadSprites();					//Dcharge les sprites
	DDClose();							//Ferme DDraw
	g_bFullScreen = !g_bFullScreen;		//Change la varible
	SaveConfig();						//Sauve la config
	//Reconfigure la fenetre
	if( g_bFullScreen )
	{
		SetWindowStyle( g_hWnd, WS_POPUP );		//Enleve la barre de titre
		SetWindowText( g_hWnd, "" );			//Et le titre
		ShowWindow( g_hWnd, SW_SHOWMAXIMIZED );	//Maximize la fenetre
	}
	else
	{
		SetWindowStyle( g_hWnd, WS_SYSMENU | WS_CAPTION | WS_MINIMIZEBOX );	//Met une barre de titre
		SetWindowText( g_hWnd, TITLE );										//Met un titre
		ShowWindow( g_hWnd, SW_RESTORE );									//Dmaximize la fenetre
		SizeAndCenterWindow( g_hWnd, RES_X, RES_Y );						//Change la taille de la fenetre et centre celle-ci
	}
	return 
		DDInit() &&						//reInitialise DDraw
		LoadCurrentSprites();			//reCharge les sprites
}


// Gestion des messages de Windows
long FAR PASCAL WindowProc( HWND hWnd, UINT message, 
                            WPARAM wParam, LPARAM lParam )
{
	switch( message )
	{
	// Activation/Dsactivation de la fenetre
	case WM_ACTIVATE:
		if(LOWORD(wParam)==WA_INACTIVE)
		{
			g_bActive = FALSE;
			InvalidateRect( hWnd, NULL, TRUE );
			DIUnacquire();
		}
		else
		{
			g_bActive = TRUE;
			DIAcquire();
		}
		return 0;
	
	// Dplacement de la fenetre
	case WM_MOVE:
		// get the client rectangle
		if ( g_bFullScreen )
		{
			SetRect( &g_rcWindow, 0, 0, RES_X, RES_Y );
		}
		else
		{
			GetClientRect( hWnd, &g_rcWindow );
			ClientToScreen( hWnd, (LPPOINT)&g_rcWindow );
			ClientToScreen( hWnd, (LPPOINT)&g_rcWindow+1 );
		}
		break;
	
	case WM_SYSKEYUP:
		switch( wParam )
		{
		// handle ALT+ENTER ( fullscreen/windowed switch )
		case VK_RETURN:
			SwitchDisplayMode();
			break;
		}
		break;
	
	// Enlve le curseur
	case WM_SETCURSOR:
		SetCursor( NULL );
		break;
	
	// Destruction de la fenetre => fin du programme
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return 0;
	}
	
	return DefWindowProc(hWnd, message, wParam, lParam);
}


static void DebugBegin(void)
{
	SYSTEMTIME st;
	
	GetLocalTime(&st);
	Debug("Program started (%i/%i/%i %i:%i:%i)",st.wMonth,st.wDay,st.wYear,
		st.wHour,st.wMinute,st.wSecond);
	Debug("Report bugs to <antikons@ifrance.com>");
}


static void DebugEnd(void)
{
	SYSTEMTIME st;
	
	GetLocalTime(&st);
	Debug("Program correctly terminated (%i/%i/%i %i:%i:%i)",st.wMonth,st.wDay,st.wYear,
		st.wHour,st.wMinute,st.wSecond);
}


int WINAPI WinMain(HINSTANCE h,
				   HINSTANCE p,
				   LPSTR cmdlgn,
				   int show)
{
	WNDCLASS	wc;					//The window class
	MSG			msg;				//Les messages de windows
	DWORD		dwStartFrame = 0;	//Pour compter le temps
	
	DebugBegin();

	g_hInstance = h;
	
	ShowLoadingWindow(h);	//Affiche la fenetre "loading..."
	
	// Charge les options de configuration
	LoadConfig();
	
	// Initialise les skins (liste les rpertoires de ./SKINS/*)
	SkinsInit();

	/** Cre une fenetre **/
	
	// Set up and register window class
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = WindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = h;
	wc.hIcon = LoadIcon( h, IDI_APPLICATION );
	wc.hCursor = LoadCursor( h, IDC_ARROW );
	wc.hbrBackground = NULL;
	wc.lpszMenuName = NULL;
	wc.lpszClassName = NAME;
	RegisterClass( &wc );
	
	// Create a window
	if( g_bFullScreen )
		g_hWnd = CreateWindow(
				NAME,
				"",
				WS_POPUP,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				640,
				480,
				NULL,
				NULL,
				h,
				NULL );
	else
		g_hWnd = CreateWindow(
				NAME,
				TITLE,
				WS_SYSMENU | WS_CAPTION | WS_MINIMIZEBOX,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				640,
				480,
				NULL,
				NULL,
				h,
				NULL );
	
	//Do we have a window?  if not exit
	if( !g_hWnd )
	{
		SkinsClose();
		Error("WinMain : CreateWindow --> Aborting");
		return -1;
	}
	
	/** Initialisations **/
		
	if(!DSInit())					//DirectSound
	{
		//Ferme tout en cas d'chec
		SkinsClose();
		Error("WinMain : DSInit --> Aborting");
		MessageBox( g_hWnd, "Can't initialize DirectSound", "ERROR", MB_OK | MB_ICONSTOP );
		return -2;
	}
	
	//Centre la fenetre
	SizeAndCenterWindow( g_hWnd, RES_X, RES_Y );	
	
	// Montre la fenetre
	if( g_bFullScreen )
		ShowWindow( g_hWnd, SW_SHOWMAXIMIZED );
	else
		ShowWindow( g_hWnd, SW_SHOW );
	
	if(!DDInit())					//DirectDraw
	{
		SkinsClose();
		DSClose();
		Error("WinMain : DDInit --> Aborting");
		MessageBox( g_hWnd, "Can't initialize DirectDraw", "ERROR", MB_OK | MB_ICONSTOP );
		return -3;
	}
	
	if(!DIInit( h, g_hWnd ))		//DirectInput
	{
		SkinsClose();
		DDClose();
		DSClose();
		Error("WinMain : DIInit --> Aborting");
		MessageBox( g_hWnd, "Can't initialize DirectInput", "ERROR", MB_OK | MB_ICONSTOP );
		return -4;
	}
	
	if(!LoadSkin(NULL))				//Charge le skin par default
	{
		SkinsClose();
		DDClose();
		DSClose();
		DIClose();
		Error("WinMain : LoadSkin --> Aborting");
		MessageBox( g_hWnd, "Can't load default skin", "ERROR", MB_OK | MB_ICONSTOP );
		return -5;
	}
		
	CloseLoadingWindow();	// Ferme la fentre "loading..."
	
	//Infinite look for message loop
	/*
	 * La boucle des messages de Windows: pour une meilleur gestion du jeu, l'cran
	 * n'est pas rafraichit  intervalles rguliers, mais ds que le programme n'a
	 * rien d'autre  faire (c'est--dire trs souvent) par un appel  DDUpdate()
	 * Quand au jeu, la fonction GameLoop() est appelle  intervalles rguliers
	 * afin d'obtenir des dplacements  vitesse rgulire
	 */
	while( 1 )
	{
		// Regarde si un message est en attente
    	if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
    	{
			// Gre le message Windows
        	if( !GetMessage( &msg, NULL, 0, 0 ) )
        	{
				// Fin du programme demande (par PostQuitMessage) => sortie de la boucle
				break;
        	}
        	TranslateMessage(&msg);
        	DispatchMessage(&msg);
    	}
		else	// Lorsque qu'aucun message n'est  traiter
		{
			// Si la fenetre est active, on rafrachit l'cran
			if( g_bActive )
				DDUpdate();
			DIProcessKBInput();
		}
		// Appelle la fonction GameLoop() toutes les g_uTimer millisecondes
		if( g_bActive )	//Si la fenetre est active
		{
			if( GetTickCount() - dwStartFrame > g_uTimer )
			{
				dwStartFrame = GetTickCount();
				if(!g_bPause)	//Si le jeu n'est pas en pause
					GameLoop();
			}
		}
	}
	
	SaveConfig();					// Sauve la configuration
	UnloadSprites();				// Dcharge les sprites
	UnloadSounds();					// Dcharge les sons
	SkinsClose();					// Ferme le skin broswer
	DDClose();						// Ferme DD
	DSClose();						// Ferme DS
	DIClose();						// Ferme DI
	UnregisterClass( NAME, h );		// Dsenregistre la classe de la fenetre
	
	DebugEnd();
	
	return 0;
}
