/* metrics.c : la taille et la disposition des diffrents sprites (pour les skins) */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <windows.h>
#include <stdio.h>
#include "main.h"
#include "../resources/resource.h"


#define DEFAULT_BLOCK_W		24
#define DEFAULT_BLOCK_H		24
#define DEFAULT_BLOCKS_X	200
#define DEFAULT_BLOCKS_Y	24
#define DEFAULT_NEXT_X		465
#define DEFAULT_NEXT_Y		55
#define DEFAULT_NUMBER_W	9
#define DEFAULT_NUMBER_H	11
#define DEFAULT_SCORE_X		130
#define DEFAULT_SCORE_Y		396
#define DEFAULT_LINES_X		130
#define DEFAULT_LINES_Y		415
#define DEFAULT_BEST_X		130
#define DEFAULT_BEST_Y		440
#define DEFAULT_LEVEL_X		242
#define DEFAULT_LEVEL_Y		465
#define DEFAULT_SPEED_X		380
#define DEFAULT_SPEED_Y		465
#define DEFAULT_PAUSE_X		39
#define DEFAULT_PAUSE_Y		154
#define DEFAULT_PAUSE_W		58
#define DEFAULT_PAUSE_H		16

int g_metricsBlockW,
	g_metricsBlockH,
	g_metricsBlockX,
	g_metricsBlockY,
	g_metricsNextX,
	g_metricsNextY,
	g_metricsNumberW,
	g_metricsNumberH,
	g_metricsScoreX,
	g_metricsScoreY,
	g_metricsLinesX,
	g_metricsLinesY,
	g_metricsBestX,
	g_metricsBestY,
	g_metricsLevelX,
	g_metricsLevelY,
	g_metricsSpeedX,
	g_metricsSpeedY,
	g_metricsPauseX,
	g_metricsPauseY,
	g_metricsPauseW,
	g_metricsPauseH;


int *getMetricPointer(const char *begin, const char *end)
{
	int i;
	char *metricsNameList[] =
	{
		"BlockW",
		"BlockH",
		"BlockX",
		"BlockY",
		"NextX",
		"NextY",
		"NumberW",
		"NumberH",
		"ScoreX",
		"ScoreY",
		"LinesX",
		"LinesY",
		"BestX",
		"BestY",
		"LevelX",
		"LevelY",
		"SpeedX",
		"SpeedY",
		"PauseX",
		"PauseY",
		"PauseW",
		"PauseH"
	};
	int *metricsPointerList[] = 
	{
		&g_metricsBlockW,
		&g_metricsBlockH,
		&g_metricsBlockX,
		&g_metricsBlockY,
		&g_metricsNextX,
		&g_metricsNextY,
		&g_metricsNumberW,
		&g_metricsNumberH,
		&g_metricsScoreX,
		&g_metricsScoreY,
		&g_metricsLinesX,
		&g_metricsLinesY,
		&g_metricsBestX,
		&g_metricsBestY,
		&g_metricsLevelX,
		&g_metricsLevelY,
		&g_metricsSpeedX,
		&g_metricsSpeedY,
		&g_metricsPauseX,
		&g_metricsPauseY,
		&g_metricsPauseW,
		&g_metricsPauseH
	};

	for(i=0;i<sizeof(metricsPointerList)/sizeof(int*);i++)
	{
		if(!_strnicmp(metricsNameList[i],begin,end-begin))
		{
			return metricsPointerList[i];
		}
	}

	return NULL;
}

void EvaluateLine(char *begin, char *end)
{
	char *p;
	int *pMetric;
	
	for( p = begin ; (p < end)&&(*p != 0) ; p++ )
	{
		if((*p==' ')||(*p=='=')||(*p=='\t'))
		{
			pMetric = getMetricPointer(begin,p);
			if(pMetric!=NULL)
			{
				for(;(p<end)&&(*p!=0);p++)
				{
					if((*p!=' ')&&(*p!='\t')&&(*p!='='))
					{
						*pMetric = atoi(p);
						return;
					}
				}
			}
			return;
		}
	}
}

void EvaluateMetricsFile(char *buf)
{
	char *prev, *next, *p;
	
	for( prev = buf, next = strchr(buf,'\n') ; next != NULL ; next = strchr(prev,'\n') )
	{
		EvaluateLine(prev,next);
		prev = next+1;
	}

	for(p=prev;*p!=0;p++);
	next = p;
	EvaluateLine(prev,next);
}


int ReadMetrics( char *filename )
{
	HRSRC res;
	HGLOBAL hResData;
	FILE *f;
	char *buf;
	int r;
	
	res = FindResource(g_hInstance, filename, "INI");
	if(res == NULL)
	{
		f = fopen(filename,"rt");
		if(!f)
			return 0;
		buf = (char*)malloc(4096);
		r = fread(buf,1,4096,f);
		fclose(f);
		if(r<1)
		{
			free(buf);
			return 0;
		}
		buf[r] = 0;
		EvaluateMetricsFile(buf);
		free(buf);
		return 1;
	}
	else
	{
		hResData = LoadResource(g_hInstance,res);
		if(hResData == NULL)
			return 0;
		buf = (char*)LockResource(hResData);
		EvaluateMetricsFile(buf);
		return 1;
	}
}


void LoadMetrics(char *path)
{
	char buf[MAX_PATH];

	//Load defaults
	g_metricsBlockW = DEFAULT_BLOCK_W	  ;
	g_metricsBlockH = DEFAULT_BLOCK_H	   ;
	g_metricsBlockX = DEFAULT_BLOCKS_X		;
	g_metricsBlockY = DEFAULT_BLOCKS_Y		 ;
	g_metricsNextX =  DEFAULT_NEXT_X		  ;
	g_metricsNextY =  DEFAULT_NEXT_Y		   ;
	g_metricsNumberW =DEFAULT_NUMBER_W			;
	g_metricsNumberH =DEFAULT_NUMBER_H 	 ;
	g_metricsScoreX = DEFAULT_SCORE_X	;
	g_metricsScoreY = DEFAULT_SCORE_Y  ;
	g_metricsLinesX = DEFAULT_LINES_X ;
	g_metricsLinesY = DEFAULT_LINES_Y;
	g_metricsBestX =  DEFAULT_BEST_X;
	g_metricsBestY =  DEFAULT_BEST_Y;
	g_metricsLevelX = DEFAULT_LEVEL_X;
	g_metricsLevelY = DEFAULT_LEVEL_Y;
	g_metricsSpeedX = DEFAULT_SPEED_X;
	g_metricsSpeedY = DEFAULT_SPEED_Y;
	g_metricsPauseX = DEFAULT_PAUSE_X	 ;
	g_metricsPauseY = DEFAULT_PAUSE_Y	;
	g_metricsPauseW = DEFAULT_PAUSE_W ;
	g_metricsPauseH = DEFAULT_PAUSE_H;
	if(path==NULL)
		ReadMetrics( MAKEINTRESOURCE(IDR_METRICS) );
	else
	{
		strcpy(buf,path);
		strcat(buf,"\\metrics.ini");
		ReadMetrics( buf );
	}
}
