/* skins.c : gestion des skins */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <malloc.h>
#include <windows.h>
#include "sounds.h"
#include "sprites.h"
#include "metrics.h"
#include "debug.h"


static char **skinList;
static int cur_skin,num_skin;


BOOL LoadSkin(char *path)
{
	UnloadSounds();
	UnloadSprites();
	
	if(!LoadSounds(path))
	{
		if(path==NULL)
		{
			Error("LoadSkin(DEFAULT) : LoadSound : The app won't use sounds");
		}
		else
		{
			Error("LoadSkin(%s):LoadSounds",path);
			LoadSounds(NULL);
		}
	}
	if(!LoadSprites(path))
	{
		if(path==NULL)
			return 0;
		else
		{
			Error("LoadSkin(%s):LoadSprites",path);
			LoadSprites(NULL);
		}
	}
	LoadMetrics(path);
	
	return 1;
}


void SkinsNext(void)
{
	if(cur_skin<num_skin-1)
		cur_skin++;
	else
		cur_skin = 0;
	LoadSkin(skinList[cur_skin]);
}


void SkinsPrev(void)
{
	if(cur_skin>0)
		cur_skin--;
	else
		cur_skin = num_skin - 1;
	LoadSkin(skinList[cur_skin]);
}


void SkinsInit(void)
{
	WIN32_FIND_DATA wfd;
	HANDLE h;
	int i;
	
	h = FindFirstFile("SKINS",&wfd);
	if(h==INVALID_HANDLE_VALUE)
	{
		skinList = (char**)malloc(sizeof(char*));
		skinList[0] = NULL;
		cur_skin = 0;
		num_skin = 1;
	}
	else
	{
		FindClose(h);
		for(h = FindFirstFile("SKINS\\*.*",&wfd),i=0;FindNextFile(h,&wfd);)
		{
			if(
				(!lstrcmp(wfd.cFileName,"."))||
				(!lstrcmp(wfd.cFileName,".."))||
				(!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
				)
				continue;
			i++;
		}
		FindClose(h);
		i++;
		num_skin = i;
		skinList = (char**)malloc(sizeof(char*)*i);
		skinList[0] = NULL;
		for(h = FindFirstFile("SKINS\\*.*",&wfd),i=1;FindNextFile(h,&wfd);)
		{
			if(
				(!lstrcmp(wfd.cFileName,"."))||
				(!lstrcmp(wfd.cFileName,".."))||
				(!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
				)
				continue;
			skinList[i] = (char*)malloc(strlen(wfd.cFileName)+7);
			strcpy(skinList[i],"SKINS\\");
			strcat(skinList[i],wfd.cFileName);
			i++;
		}
		cur_skin = 0;
	}
}


void SkinsClose(void)
{
	int i;
	
	for(i=0;i<num_skin;i++)
	{
		free(skinList[i]);
	}
	
	free(skinList);
}


BOOL LoadCurrentSprites(void)
{
	if(!LoadSprites(skinList[cur_skin]))
		return LoadSprites(NULL);
	return TRUE;
}
