/* sounds.c : Gestion des sons du jeu */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <dsound.h>
#include "dsutil.h"
#include "../resources/resource.h"
#include "debug.h"


LPDIRECTSOUNDBUFFER g_lpDSBMove = NULL;
LPDIRECTSOUNDBUFFER g_lpDSBTurn = NULL;
LPDIRECTSOUNDBUFFER g_lpDSBDestroy = NULL;
LPDIRECTSOUNDBUFFER g_lpDSBDrop = NULL;
BOOL g_bSounds = FALSE;						// Jeu muet?


__inline void ReleaseSoundBuffer( LPDIRECTSOUNDBUFFER *lplpDSB )
{
	if( *lplpDSB )
	{
		IDirectSoundBuffer_Release( *lplpDSB );
		*lplpDSB = NULL;
	}
}


void UnloadSounds(void)
{
	ReleaseSoundBuffer( &g_lpDSBMove );
	ReleaseSoundBuffer( &g_lpDSBTurn );
	ReleaseSoundBuffer( &g_lpDSBDestroy );
	ReleaseSoundBuffer( &g_lpDSBDrop );
}


// Charge tous les sons
BOOL LoadSounds(char *path)
{
	char buf[MAX_PATH];
	
	// MOVE
	if(path==NULL)
	{
		//Load as wave resource
		g_lpDSBMove = DSLoadWave( MAKEINTRESOURCE(IDR_MOVE) );
	}
	else
	{
		//Load as Mpeg
		strcpy(buf,path);
		strcat(buf,"\\move.mp3");
		g_lpDSBMove = DSLoadMpeg( buf );
		if(!g_lpDSBMove)
		{
			//Load as Wave
			strcpy(buf,path);
			strcat(buf,"\\move.wav");
			g_lpDSBMove = DSLoadWave( buf );
		}
	}
	if( g_lpDSBMove == NULL )
	{
		Error("LoadSounds : g_lpDSBMove(%s)",path);
		return FALSE;
	}
	
	// TURN
	if(path==NULL)
	{
		//Load as wave resource
		g_lpDSBTurn = DSLoadWave( MAKEINTRESOURCE(IDR_TURN) );
	}
	else
	{
		//Load as Mpeg
		strcpy(buf,path);
		strcat(buf,"\\turn.mp3");
		g_lpDSBTurn = DSLoadMpeg( buf );
		if(!g_lpDSBTurn)
		{
			//Load as wave
			strcpy(buf,path);
			strcat(buf,"\\turn.wav");
			g_lpDSBTurn = DSLoadWave( buf );
		}
	}
	if( g_lpDSBTurn == NULL )
	{
		Error("LoadSounds : g_lpDSBTurn(%s)",path);
		return FALSE;
	}
	
	// DESTROY
	if(path==NULL)
	{
		//Load as wave resource
		g_lpDSBDestroy = DSLoadWave( MAKEINTRESOURCE(IDR_DESTROY) );
	}
	else
	{
		//Load as Mpeg
		strcpy(buf,path);
		strcat(buf,"\\destroy.mp3");
		g_lpDSBDestroy = DSLoadMpeg( buf );
		if(!g_lpDSBDestroy)
		{
			//Load as wave
			strcpy(buf,path);
			strcat(buf,"\\destroy.wav");
			g_lpDSBDestroy = DSLoadWave( buf );
		}
	}
	if( g_lpDSBDestroy == NULL )
	{
		Error("LoadSounds : g_lpDSBDestroy(%s)",path);
		return FALSE;
	}
	
	// DROP
	if(path==NULL)
	{
		//Load as wave resource
		g_lpDSBDrop = DSLoadWave( MAKEINTRESOURCE(IDR_DROP) );
	}
	else
	{
		//Load as Mpeg
		strcpy(buf,path);
		strcat(buf,"\\drop.mp3");
		g_lpDSBDrop = DSLoadMpeg( buf );
		if(!g_lpDSBDrop)
		{
			//Load as wave
			strcpy(buf,path);
			strcat(buf,"\\drop.wav");
			g_lpDSBDrop = DSLoadWave( buf );
		}
	}
	if( g_lpDSBDrop == NULL )
	{
		Error("LoadSounds : g_lpDSBDrop(%s)",path);
		return FALSE;
	}
	
	return TRUE;
}


// Joue le son
void PlayMove(void)
{
	if( g_bSounds )
	{
		if(!DSPlay( g_lpDSBMove ))
			Error("PlayMove");
	}
}


// Joue le son
void PlayTurn(void)
{
	if( g_bSounds )
	{
		if(!DSPlay( g_lpDSBTurn ))
			Error("PlayTurn");
	}
}


// Joue le son
void PlayDestroy(void)
{
	if( g_bSounds )
	{
		if(!DSPlay( g_lpDSBDestroy ))
			Error("PlayDestroy");
	}
}


// Joue le son
void PlayDrop(void)
{
	if( g_bSounds )
	{
		if(!DSPlay( g_lpDSBDrop ))
			Error("PlayDrop");
	}
}
