/* sprites.c : gestion des sprites */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * UltimateTetris                           Copyright (C) 2001, AnTiKoNs *
 *----------------/                                                      *
 * This program is free software; you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * The author of this program may be contacted at antikons@ifrance.com   *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <ddraw.h>
#include "ddutil.h"
#include "../resources/resource.h"
#include "debug.h"


LPDIRECTDRAWSURFACE g_lpDDSBlocks[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
LPDIRECTDRAWSURFACE g_lpDDSNumber[10] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };	// Les nombres de 0  9
LPDIRECTDRAWSURFACE g_lpDDSBackground = NULL;
LPDIRECTDRAWSURFACE g_lpDDSPause = NULL;



BOOL LoadNumbers(char *path)
{
	char buf[MAX_PATH];
	int i;
	int IDB_NUMBERS[10] = { IDB_NUMBER0,
							IDB_NUMBER1,
							IDB_NUMBER2,
							IDB_NUMBER3,
							IDB_NUMBER4,
							IDB_NUMBER5,
							IDB_NUMBER6,
							IDB_NUMBER7,
							IDB_NUMBER8,
							IDB_NUMBER9
	};
	for( i = 0 ; i < 10 ; i++ )
	{
		if(path==NULL)
		{
			//Load default skin bitmap
			g_lpDDSNumber[i] = DDLoadBitmap( MAKEINTRESOURCE(IDB_NUMBERS[i]), 0, 0 );
		}
		else
		{
			//Load as BMP
			wsprintf(buf,"%s\\number%i.bmp",path,i);
			g_lpDDSNumber[i] = DDLoadBitmap( buf, 0, 0 );
			if( g_lpDDSNumber[i] == NULL )
			{
				//Load as JPG
				wsprintf(buf,"%s\\number%i.jpg",path,i);
				g_lpDDSNumber[i] = DDLoadJpeg( buf );
			}
		}
		if( g_lpDDSNumber[i] == NULL )
			return FALSE;
	}
	
	return TRUE;
}


BOOL LoadBlocks(char *path)
{
	char buf[MAX_PATH];
	int i;
	int IDR_BLOCKS[7] = { IDR_BLOCK1,
							IDR_BLOCK2,
							IDR_BLOCK3,
							IDR_BLOCK4,
							IDR_BLOCK5,
							IDR_BLOCK6,
							IDR_BLOCK7
	};
	for( i = 0 ; i < 7 ; i++ )
	{
		if(path==NULL)
		{
			//Load default skin jpeg
			g_lpDDSBlocks[i] = DDLoadJpeg( MAKEINTRESOURCE(IDR_BLOCKS[i]) );
		}
		else
		{
			//Load as JPG
			wsprintf(buf,"%s\\block%i.jpg",path,i+1);
			g_lpDDSBlocks[i] = DDLoadJpeg( buf );
			if( g_lpDDSBlocks[i] == NULL )
			{
				//Load as BMP
				wsprintf(buf,"%s\\block%i.bmp",path,i+1);
				g_lpDDSBlocks[i] = DDLoadBitmap( buf, 0, 0 );
			}
		}
		if( g_lpDDSBlocks[i] == NULL )
			return FALSE;
	}
	
	return TRUE;
}


// Charge tous les sprites
BOOL LoadSprites(char *path)
{
	char buf[MAX_PATH];

	if(!LoadNumbers(path))
		return FALSE;
	if(!LoadBlocks(path))
		return FALSE;
	
	// Pause
	if(path==NULL)
	{
		//Load default skin bitmap
		g_lpDDSPause = DDLoadBitmap( MAKEINTRESOURCE(IDB_PAUSE), 0, 0 );
	}
	else
	{
		//Load as BMP
		strcpy(buf,path);
		strcat(buf,"\\pause.bmp");
		g_lpDDSPause = DDLoadBitmap( buf, 0, 0 );
		if(g_lpDDSPause == NULL)
		{
			//Load as JPG
			strcpy(buf,path);
			strcat(buf,"\\pause.jpg");
			g_lpDDSPause = DDLoadJpeg( buf );
		}
	}
	if(g_lpDDSPause == NULL)
		return FALSE;

	// Background
	if(path==NULL)
	{
		g_lpDDSBackground = DDLoadJpeg( MAKEINTRESOURCE(IDR_BACKGROUND) );
	}
	else
	{
		//Load as JPG
		strcpy(buf,path);
		strcat(buf,"\\background.jpg");
		g_lpDDSBackground = DDLoadJpeg( buf );
		if(g_lpDDSBackground == NULL)
		{
			//Load as BMP
			strcpy(buf,path);
			strcat(buf,"\\background.bmp");
			g_lpDDSBackground = DDLoadBitmap( buf, 0, 0 );
		}
	}
	if(g_lpDDSBackground == NULL)
		return FALSE;
	
	return TRUE;
}


void UnloadNumbers(void)
{
	int i;

	for( i = 0 ; i < 10 ; i++ )
	{
		if( g_lpDDSNumber[i] )
		{
			IDirectDrawSurface_Release( g_lpDDSNumber[i] );
			g_lpDDSNumber[i] = NULL;
		}
	}
}


void UnloadBlocks(void)
{
	int i;

	for( i = 0 ; i < 7 ; i++ )
	{
		if( g_lpDDSBlocks[i] )
		{
			IDirectDrawSurface_Release( g_lpDDSBlocks[i] );
			g_lpDDSBlocks[i] = NULL;
		}
	}
}


void UnloadSprites(void)
{
	UnloadNumbers();
	UnloadBlocks();
	if( g_lpDDSPause )
	{
		IDirectDrawSurface_Release( g_lpDDSPause );
		g_lpDDSPause = NULL;
	}
	if( g_lpDDSBackground )
	{
		IDirectDrawSurface_Release( g_lpDDSBackground );
		g_lpDDSBackground = NULL;
	}
}
